package com.saini.testingchat.manager;

import java.io.IOException;
import java.util.Collection;

import org.jivesoftware.smack.AbstractXMPPConnection;
import org.jivesoftware.smack.ConnectionConfiguration.SecurityMode;
import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.SASLAuthentication;
import org.jivesoftware.smack.SmackException;
import org.jivesoftware.smack.SmackException.NotConnectedException;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.chat.Chat;
import org.jivesoftware.smack.chat.ChatManager;
import org.jivesoftware.smack.chat.ChatManagerListener;
import org.jivesoftware.smack.filter.StanzaFilter;
import org.jivesoftware.smack.filter.StanzaTypeFilter;
import org.jivesoftware.smack.packet.DefaultExtensionElement;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Presence;
import org.jivesoftware.smack.packet.Presence.Type;
import org.jivesoftware.smack.packet.Stanza;
import org.jivesoftware.smack.roster.Roster;
import org.jivesoftware.smack.roster.Roster.SubscriptionMode;
import org.jivesoftware.smack.roster.RosterEntry;
import org.jivesoftware.smack.roster.RosterListener;
import org.jivesoftware.smack.tcp.XMPPTCPConnection;
import org.jivesoftware.smack.tcp.XMPPTCPConnectionConfiguration;

import android.annotation.SuppressLint;
import android.os.StrictMode;
import android.util.Log;

import com.saini.testingchat.util.ChatContants;

@SuppressLint("NewApi")
public class XmppManager {
	private String TAG = "XmppManager";
	private XMPPTCPConnectionConfiguration connConfig;
	private static AbstractXMPPConnection connection;
	private static Roster roster;
	private ChatManager chatManager;

	public XmppManager() {

	}

	public boolean init() {
		boolean isConnectionFIne = false;
		Log.d(TAG, String.format(
				"Connection is null in INIT XMPPMANAGER: %1$s port %2$d",
				ChatContants.SERVERNAME, ChatContants.PORT));
		if (android.os.Build.VERSION.SDK_INT > 9) {
			StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder()
					.permitAll().build();
			StrictMode.setThreadPolicy(policy);
		}
		connConfig = XMPPTCPConnectionConfiguration.builder()
				.setConnectTimeout(5000).setHost(ChatContants.SERVERNAME)
				.setPort(ChatContants.PORT).setSendPresence(true)
				.setServiceName(ChatContants.SERVICENAME)
				.setDebuggerEnabled(false)
				.setSecurityMode(SecurityMode.disabled).build();
		SASLAuthentication.blacklistSASLMechanism("SCRAM-SHA-1");
		SASLAuthentication.blacklistSASLMechanism("DIGEST-MD5");
		SASLAuthentication.unBlacklistSASLMechanism("PLAIN");
		connection = new XMPPTCPConnection(connConfig);

		try {
			connection.connect();
			isConnectionFIne = connection.isConnected();
			Log.d(TAG,
					"Connection after Connect method in INIT XMPPMANAGER | Connected is: "
							+ connection.isConnected());

			chatManager = ChatManager.getInstanceFor(connection);
		} catch (XMPPException e) {
			isConnectionFIne = false;
			// TODO Auto-generated catch block
			e.printStackTrace();

		} catch (SmackException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return isConnectionFIne;
	}

	public boolean performLogin(String username, String password) {
		boolean isLoginDone = false;
		if (connection != null && connection.isConnected()) {

			try {
				connection.login(username, password);
				Log.i(TAG, "Logged in as: " + connection.getUser());

				// Set the status to available
				setStatus(true, "I am Online...");
				roster = Roster.getInstanceFor(connection);
				Roster.setDefaultSubscriptionMode(SubscriptionMode.accept_all);
				setRoster(roster);
				roster.addRosterListener(new RosterListener() {
					// Ignored events public void
					// entriesAdded(Collection<String> addresses) {}
					public void entriesDeleted(Collection<String> addresses) {
					}

					public void entriesUpdated(Collection<String> addresses) {
					}

					public void presenceChanged(Presence presence) {
						Log.d(TAG,
								"Connection is null in performLogin XMPPMANAGER Presence changed: "
										+ presence.getTo() + " " + presence);
					}

					@Override
					public void entriesAdded(Collection<String> arg0) {
						// TODO Auto-generated method stub

						Log.d(TAG,
								"entry is added to the roster in performLogin entries addded: "
										+ arg0.size() + " "
										+ arg0.toArray()[0].toString());
					}
				});
				isLoginDone = true;
				Log.d(TAG, "Pta hai " + connection.getUser());
			} catch (XMPPException e) {
				isLoginDone = false;
				e.printStackTrace();
			} catch (Exception e) {
				isLoginDone = false;
				e.printStackTrace();
			}
		}
		return isLoginDone;
	}

	public void setStatus(boolean available, String status) {

		Presence.Type type = available ? Type.available : Type.unavailable;
		Presence presence = new Presence(type);

		presence.setStatus(status);
		try {
			connection.sendStanza(presence);
		} catch (NotConnectedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void printRoster() {
		Log.d(TAG, "IN PrintRsetr method");

		Collection<RosterEntry> entries = roster.getEntries();
		Log.d(TAG, "Roster USers: " + entries.size());

		for (RosterEntry entry1 : entries) {
			Log.d(TAG,
					"\tUserID:- " + entry1.getUser() + "\tName:- "
							+ entry1.getName() + "\tStatus:- "
							+ entry1.getStatus() + "\ttype:- "
							+ entry1.getType());
		}

	}

	public void destroy() {
		if (connection != null && connection.isConnected()) {

			Presence offlinePres = new Presence(Presence.Type.unavailable,
					connection.getUser() + " Bye", 1, Presence.Mode.away);
			try {
				connection.sendStanza(offlinePres);
				Log.d(TAG,
						"Connection is null in destroy User staus set as unavailable");

				connection.disconnect(offlinePres);
			} catch (Exception exception) {
				exception.printStackTrace();
				connection.disconnect();
			}
			Log.d(TAG, "Connection is null  in destroy User is CCCCconnected"
					+ connection.isConnected());

		} else {
			Log.d(TAG,
					"Connection is null in destroy Connecttion show already is disconnected");
		}
	}

	public void sendMessage(String buddyJID, String messageTe) {

		String OurBuddy = buddyJID + "@" + ChatContants.SERVICENAME + "/Spark";
		String from = connection.getUser();
		Log.d(TAG,
				String.format(
						"Connection is null in sendMessage  Sending mesage '%1$s' to user %2$s from Me %3$s   passed %4$s",
						messageTe, OurBuddy, from, buddyJID));
		if (chatManager != null) {
			Message msgObj = new Message();
			msgObj.setBody(messageTe);
			msgObj.setTo(buddyJID);
			msgObj.setFrom(from);
			msgObj.setType(Message.Type.chat);
			msgObj.addExtension(new DefaultExtensionElement("from", from));
			msgObj.addExtension(new DefaultExtensionElement("to", buddyJID));
			try {
				connection.sendStanza(msgObj);
			} catch (NotConnectedException e) {
				// TODO Auto-generated catch block
				Log.d(TAG, "sendStanza is creating some problem");
				e.printStackTrace();
			}

		} else {
			Log.d(TAG, "XMPPMANGER  sendMEssage chamAnaeger is Null");
		}
	}

	public void registerIncomingMessageListener() {
		// Assume we've created an XMPPConnection name "connection"._
		if (chatManager != null) {
			chatManager.addChatListener(new ChatManagerListener() {

				@Override
				public void chatCreated(Chat chat, boolean createdLocally) {
					Log.d(TAG,
							"registerIncomingMessageListener Chat is created for you "
									+ chat.getParticipant() + "|" + chat);
					if (!createdLocally)
						chat.addMessageListener(new MyNewMessageListener());
				}
			});
		} else {
			Log.d(TAG,
					"XMPPMANGER  registerIncomingMessageListener chamAnaeger is Null");
		}
	}

	@SuppressWarnings("deprecation")
	public void registerIncomingPackateListener() {
		// Assume we've created an XMPPConnection name "connection"._
		StanzaFilter filter = new StanzaTypeFilter(Message.class);
		PacketListener myListener = new PacketListener() {

			@Override
			public void processPacket(Stanza stanza)
					throws NotConnectedException {
				// TODO Auto-generated method stub
				if (stanza instanceof Message) {
					Log.d(TAG, "Recieve Packate" + ((Message) stanza).getBody()
							+ "|" + ((Message) stanza).getFrom());
					sendMessage(((Message) stanza).getFrom(),
							"Reply From Automation");
				} else {
					Log.d(TAG, "Recieve Packate unknown");
				}
			}

		};
		// Register the listener._
		connection.addPacketListener(myListener, filter);
	}

	public AbstractXMPPConnection getConnection() {
		return connection;
	}

	public void setConnection(AbstractXMPPConnection connection) {
		this.connection = connection;
	}

	public Roster getRoster() {
		return roster;
	}

	public void setRoster(Roster roster) {
		this.roster = roster;
	}

}